﻿/**
Copyright(c)2008 nutsu.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/**
* ColorBlend
* @author nutsu
* @version 0.1
*/

package com.nutsu.color {
	
	public class ColourBlend {
		
		public static const ADD:String 			= "add";
		public static const SUBTRACT:String 	= "subtract";
		public static const DARKEN:String 		= "darken";
		public static const LIGHTEN:String 		= "lighten";
		public static const DIFFERENCE:String 	= "difference"
		public static const MULTIPLY:String 	= "multiply";
		public static const SCREEN:String 		= "screen";
		public static const OVERLAY:String 		= "overlay";
		public static const HARDLIGHT:String 	= "hardlight";
		public static const SOFTLIGHT:String 	= "softlight";
		public static const DODGE:String		= "dodge";
		public static const BURN:String			= "burn";
		public static const EXCLUSION:String	= "exclusion";
		
		
		public function ColourBlend() {
			throw new Error("インスタンスはつくっても意味ないです");
		}
		
		/**
		 * blend
		 * @param	blendtype
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @param	alpha backcolor [0,1]
		 * @param	alpha forecolor [0,1]
		 * @return 	{value:0xRRGGBB,alpha:Number}
		 */
		public static function blend( blendkind:String, c1:uint, c2:uint, a1:Number=NaN, a2:Number=NaN ):Object
		{
			if ( (isNaN(a1) && isNaN(a2)) || ( a1==1.0 && a2==1.0 ) )
			{
				return { value:blend24( blendkind, c1, c2 ), alpha:1.0 };
			}
			else
			{
				var c:uint = blend32( blendkind, _to32(c1, a1), _to32(c2, a2) );
				return { value: c & 0x00ffffff , alpha: ( c >>> 24 ) / 0xff };
			}
		}
		
		/**
		 * blend
		 * @param	blendtype
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @param	alpha backcolor [0,1]
		 * @param	alpha forecolor [0,1]
		 * @return 	0xAARRGGBB
		 */
		public static function blendTo32( blendkind:String, c1:uint, c2:uint, a1:Number=NaN, a2:Number=NaN ):uint
		{
			if ( isNaN(a1) && isNaN(a2 ) || ( a1==1.0 && a2==1.0 )  )
				return 0xff000000 | blend24( blendkind, c1, c2 );
			else
				return blend32( blendkind, _to32(c1, a1), _to32(c2, a2) );
		}
		
		/**
		 * blend 24bit
		 * @param	blendtype
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return 	0xRRGGBB
		 */
		public static function blend24( blendkind:String, c1:uint, c2:uint ):uint
		{
			switch( blendkind )
			{
				case ADD: 			return add( c1, c2 );
				case SUBTRACT: 		return sub( c1, c2 );
				case DARKEN: 		return dark( c1, c2 );
				case LIGHTEN: 		return light( c1, c2 );
				case DIFFERENCE: 	return diff( c1, c2 );
				case MULTIPLY: 		return multi( c1, c2 );
				case SCREEN: 		return screen( c1, c2 );
				case OVERLAY: 		return overlay( c1, c2 );
				case HARDLIGHT: 	return hard( c1, c2 );
				case SOFTLIGHT: 	return soft( c1, c2 );
				case DODGE: 		return dodge( c1, c2 );
				case BURN: 			return burn( c1, c2 );
				case EXCLUSION:		return excl( c1, c2 );
				default:			return c2;
			}
		}
		
		/**
		 * blend 32bit
		 * @param	blendtype
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return 	0xAARRGGBB
		 */
		public static function blend32( blendkind:String, c1:uint, c2:uint ):uint
		{
			switch( blendkind )
			{
				case ADD: 			return add32( c1, c2 );
				case SUBTRACT: 		return sub32( c1, c2 );
				case DARKEN: 		return dark32( c1, c2 );
				case LIGHTEN: 		return light32( c1, c2 );
				case DIFFERENCE: 	return diff32( c1, c2 );
				case MULTIPLY: 		return multi32( c1, c2 );
				case SCREEN: 		return screen32( c1, c2 );
				case OVERLAY: 		return overlay32( c1, c2 );
				case HARDLIGHT: 	return hard32( c1, c2 );
				case SOFTLIGHT: 	return soft32( c1, c2 );
				case DODGE: 		return dodge32( c1, c2 );
				case BURN: 			return burn32( c1, c2 );
				case EXCLUSION:		return excl32( c1, c2 );
				default:			return c2;
			}
		}
		
		//---------------------------------------------------------------------------------------------------BLEND PRC
		/**
		 * 加算
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function add( c1:uint, c2:uint ):uint
		{
			return addRGB( (c1 & 0xff0000) >>> 16, (c1 & 0x00ff00) >>> 8, c1 & 0x0000ff, (c2 & 0xff0000) >>> 16, (c2 & 0x00ff00) >>> 8, c2 & 0x0000ff );	
		}
		public static function addRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return Math.min( r1 + r2, 0xff ) << 16 | Math.min( g1 + g2, 0xff ) << 8 | Math.min( b1 + b2, 0xff );
		}
		/**
		 * 加算 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function add32( c1:uint, c2:uint ):uint
		{
			return addRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function addRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | Math.min( r1 + (r2*a2>>8) , 0xff ) << 16 | Math.min( g1 + (g2*a2>>8), 0xff ) << 8 | Math.min( b1 + (b2*a2>>8), 0xff );
		}
		
		//-------------------------------
		/**
		 * 減算
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function sub( c1:uint, c2:uint ):uint
		{
			return subRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function subRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return Math.max( r1 - r2, 0 ) << 16 | Math.max( g1 - g2, 0 ) << 8 | Math.max( b1 - b2, 0 );
		}
		/**
		 * 減算 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function sub32( c1:uint, c2:uint ):uint
		{
			return subRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function subRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | Math.max( r1 - (r2*a2>>8) , 0 ) << 16 | Math.max( g1 - (g2*a2>>8), 0 ) << 8 | Math.max( b1 - (b2*a2>>8), 0 );
		}
		
		//-------------------------------
		/**
		 * 暗い方
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function dark( c1:uint, c2:uint ):uint
		{
			return darkRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function darkRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return Math.min( r1, r2 ) << 16 | Math.min( g1, g2 ) << 8 | Math.min( b1, b2 );
		}
		/**
		 * 暗い方 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function dark32( c1:uint, c2:uint ):uint
		{
			return darkRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function darkRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _dark32(r1,r2,a2) << 16 | _dark32(g1,g2,a2) << 8 | _dark32(b1,b2,a2);
		}
		private static function _dark32( e1:uint, e2:uint, a2:uint ):uint
		{
			return (e1*(a2^0xff)>>8) + (Math.min(e1,(e2*a2>>8))*a2>>8);
		}
		
		//-------------------------------
		/**
		 * 明るい方
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function light( c1:uint, c2:uint ):uint
		{
			return lightRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function lightRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return Math.max( r1, r2 ) << 16 | Math.max( g1, g2 ) << 8 | Math.max( b1, b2 );
		}
		/**
		 * 明るい方 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function light32( c1:uint, c2:uint ):uint
		{
			return lightRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function lightRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _light32(r1,r2,a2) << 16 | _light32(g1,g2,a2) << 8 | _light32(b1,b2,a2);
		}
		private static function _light32( e1:uint, e2:uint, a2:uint ):uint
		{
			return  Math.max( e1, (e2*a2>>8) );
		}
		//-------------------------------
		/**
		 * 差の絶対値
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function diff( c1:uint, c2:uint ):uint
		{
			return diffRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function diffRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return Math.abs( r1-r2 ) << 16 | Math.abs( g1-g2 ) << 8 | Math.abs( b1-b2 );
		}
		/**
		 * 差の絶対値 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function diff32( c1:uint, c2:uint ):uint
		{
			return diffRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function diffRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _diff32(r1,r2,a2) << 16 | _diff32(g1,g2,a2) << 8 | _diff32(b1,b2,a2);
		}
		private static function _diff32( e1:uint, e2:uint, a2:uint ):uint
		{
			return (e1*(a2^0xff)>>8) + (Math.abs(e1 - e2)*a2>>8);
		}
		
		//-------------------------------
		/**
		 * 乗算
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function multi( c1:uint, c2:uint ):uint
		{
			return multiRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function multiRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return ( r1*r2>>8 ) << 16 | ( g1*g2>>8 ) << 8 | b1*b2>>8;
		}
		/**
		 * 乗算 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function multi32( c1:uint, c2:uint ):uint
		{
			return multiRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function multiRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _multi32(r1,r2,a2) << 16 | _multi32(g1,g2,a2) << 8 | _multi32(b1,b2,a2);
		}
		private static function _multi32( e1:uint, e2:uint, a2:uint ):uint
		{
			return (e1*(a2^0xff)>>8) + (e1*e2*a2>>16);
		}
		
		//-------------------------------
		/**
		 * スクリーン
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function screen( c1:uint, c2:uint ):uint
		{
			return multi( c1 ^ 0xffffff, c2 ^ 0xffffff ) ^ 0xffffff;
		}
		public static function screenRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return multiRGB( r1^0xff, g1^0xff, b1^0xff, r2^0xff, g2^0xff, b2^0xff )^0xffffff;
		}
		/**
		 * スクリーン 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function screen32( c1:uint, c2:uint ):uint
		{
			return screenRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function screenRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | multiRGB( r1^0xff, g1^0xff, b1^0xff, (r2*a2>>8)^0xff, (g2*a2>>8)^0xff, (b2*a2>>8)^0xff )^0xffffff;
		}
		
		//-------------------------------
		/**
		 * オーバーレイ
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function overlay( c1:uint, c2:uint ):uint
		{
			return overlayRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function overlayRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return _overlay(r1,r2)<<16 | _overlay(g1,g2)<<8 | _overlay(b1,b2);
		}
		private static function _overlay( e1:uint, e2:uint ):uint
		{
			return ( e1 < 0x80 ) ? e1*e2>>7 : ((e1^0xff)*(e2^0xff)>>7)^0xff;
		}
		/**
		 * オーバーレイ 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function overlay32( c1:uint, c2:uint ):uint
		{
			return overlayRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function overlayRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _overlay32(r1,r2,a2) << 16 | _overlay32(g1,g2,a2) << 8 | _overlay32(b1,b2,a2);
		}
		private static function _overlay32( e1:uint, e2:uint, a2:uint ):uint
		{
			return (e1*(a2^0xff)>>8) + ( ( e1 < 0x80 ) ? e1*e2*a2>>15 : (((e1^0xff)*(e2^0xff)>>7)^0xff)*a2>>8 );
		}
		
		//-------------------------------
		/**
		 * ハードライト
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function hard( c1:uint, c2:uint ):uint
		{
			return hardRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function hardRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return _hard(r1,r2)<<16 | _hard(g1,g2)<<8 | _hard(b1,b2);
		}
		private static function _hard( e1:uint, e2:uint ):uint
		{
			return ( e2 < 0x80 ) ? e1*e2>>7 : ((e1^0xff)*(e2^0xff)>>7)^0xff;
		}
		/**
		 * ハードライト 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function hard32( c1:uint, c2:uint ):uint
		{
			return hardRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function hardRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _hard32(r1,r2,a2)<<16 | _hard32(g1,g2,a2)<<8 | _hard32(b1,b2,a2);
		}
		private static function _hard32( e1:uint, e2:uint, a2:uint ):uint
		{
			return (e1*(a2^0xff)>>8) + ( ( e2 < 0x80 ) ? e1*e2*a2>>15 : (((e1^0xff)*(e2^0xff)>>7)^0xff)*a2>>8 );
		}
		
		//-------------------------------
		/**
		 * ソフトライト
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function soft( c1:uint, c2:uint ):uint
		{
			return softRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function softRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return _soft(r1,r2)<<16 | _soft(g1,g2)<<8 | _soft(b1,b2);
		}
		private static function _soft( e1:uint, e2:uint ):uint
		{
			return ( e1 * e2 + e1 * ( 0xff - ((e1 ^ 0xff) * (e2 ^ 0xff) >> 8) - (e1 * e2 >> 8) )) >> 8;
		}
		
		/**
		 * ソフトライト 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function soft32( c1:uint, c2:uint ):uint
		{
			return softRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function softRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _soft32(r1,r2,a2)<<16 | _soft32(g1,g2,a2)<<8 | _soft32(b1,b2,a2);
		}
		private static function _soft32( e1:uint, e2:uint, a2:uint ):uint
		{
			return (e1*(a2^0xff)>>8) + (( e1*e2 + e1*( 0xff - ((e1^0xff)*(e2^0xff)>>8) - (e1*e2>>8) ))*a2>>16);
		}
		
		//-------------------------------
		/**
		 * 覆い焼き
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function dodge( c1:uint, c2:uint ):uint
		{
			return dodgeRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function dodgeRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return _dodge(r1,r2)<<16 | _dodge(g1,g2)<<8 | _dodge(b1,b2);
		}
		private static function _dodge( e1:uint, e2:uint ):uint
		{
			var x:uint;
			if ( e2 == 0xff || ( x = Math.floor( e1*0xff/(e2^0xff)) ) > 0xff )
				return 0xff;
			else
				return x;
		}
		/**
		 * 覆い焼き 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function dodge32( c1:uint, c2:uint ):uint
		{
			return dodgeRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function dodgeRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _dodge32(r1,r2,a2)<<16 | _dodge32(g1,g2,a2)<<8 | _dodge32(b1,b2,a2);
		}
		private static function _dodge32( e1:uint, e2:uint, a2:uint ):uint
		{
			var x:uint;
			if ( e2 == 0xff || ( x = Math.floor( e1*0xff/(e2^0xff)) ) > 0xff )
				x = 0xff;
			return (e1*(a2^0xff) + x*a2)>>8;
		}
		
		//-------------------------------
		/**
		 * 焼き込み
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function burn( c1:uint, c2:uint ):uint
		{
			return burnRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function burnRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return _burn(r1,r2)<<16 | _burn(g1,g2)<<8 | _burn(b1,b2);
		}
		private static function _burn( e1:uint, e2:uint ):uint
		{
			var x:uint;
			if ( e2 == 0 || (x = 0xff - Math.floor( (e1^0xff)*0xff/e2 ) ) < 0 )
				return 0;
			else
				return x;
		}
		/**
		 * 焼き込み 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function burn32( c1:uint, c2:uint ):uint
		{
			return burnRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function burnRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _burn32(r1,r2,a2)<<16 | _burn32(g1,g2,a2)<<8 | _burn32(b1,b2,a2);
		}
		private static function _burn32( e1:uint, e2:uint, a2:uint ):uint
		{
			var x:uint;
			if ( e2 == 0 || (x = 0xff - Math.floor( (e1^0xff)*0xff/e2 ) ) < 0 )
				x = 0;
			return (e1*(a2^0xff) + x*a2)>>8;
		}
		
		//-------------------------------
		/**
		 * 除外
		 * @param	backcolor 0xRRGGBB
		 * @param	forecolor 0xRRGGBB
		 * @return	0xRRGGBB
		 */
		public static function excl( c1:uint, c2:uint ):uint
		{
			return exclRGB( (c1 & 0xff0000)>>>16, (c1 & 0x00ff00)>>>8, c1 & 0x0000ff, (c2 & 0xff0000)>>>16, (c2 & 0x00ff00)>>>8, c2 & 0x0000ff );
		}
		public static function exclRGB( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint ):uint
		{
			return _excl(r1,r2)<<16 | _excl(g1,g2)<<8 | _excl(b1,b2);
		}
		private static function _excl( e1:uint, e2:uint ):uint
		{
			return e1 + e2 - ( e1*e2>>>7 );
		}
		/**
		 * 除外 32bit
		 * @param	backcolor 0xAARRGGBB
		 * @param	forecolor 0xAARRGGBB
		 * @return	0xAARRGGBB
		 */
		public static function excl32( c1:uint, c2:uint ):uint
		{
			return exclRGBA( (c1 & 0x00ff0000) >>> 16, (c1 & 0x0000ff00) >>> 8, c1 & 0x000000ff, (c2 & 0x00ff0000) >>> 16, (c2 & 0x0000ff00) >>> 8, c2 & 0x000000ff, (c1 & 0xff000000) >>> 24, (c2 & 0xff000000) >>> 24 );
		}
		public static function exclRGBA( r1:uint, g1:uint, b1:uint, r2:uint, g2:uint, b2:uint, a1:uint=0xff, a2:uint=0xff ):uint
		{
			return Math.min( a1 + a2, 0xff ) << 24 | _excl32(r1,r2,a2)<<16 | _excl32(g1,g2,a2)<<8 | _excl32(b1,b2,a2);
		}
		private static function _excl32( e1:uint, e2:uint, a2:uint ):uint
		{
			return e1 + (e2 * a2 >> 8) - ( e1 * e2 * a2 >>> 15 );
		}
		
		//---------------------------------------------------------------------------------------------------
		
		/**
		 * @private
		 */
		private static function _to32( c:uint, a:Number ):uint
		{
			a = ( isNaN(a) ) ? 0xff : Math.round(0xff*a);
			return (a << 24) | c;
		}
	}
}